#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json, urllib

def request(url, func, *args):
    req = json.dumps({"method":func, "params":args, "id":1})
    result = urllib.urlopen(url, req).read()
    try:
        response = json.loads(result)
    except:
        return "error: %s" % result
    else:
        return response.get("result", response.get("error"))

url = "http://localhost:8888/jsonrpc"
print "echo:", request(url, "echo", "foo bar")
print "sort:", request(url, "sort", ["foo", "bar"])
print "count:", request(url, "count", ["foo", "bar"])
print "geoip_lookup:", request(url, "geoip_lookup", "google.com")



# coding: utf-8
# Copyright 2009 Alexandre Fiori
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import types
import operator
import functools
import collections
from twisted.internet import task, defer
from cyclone.redis.hashring import HashRing
import re

from cyclone.redis import protocol

_findhash = re.compile('.+\{(.*)\}.*', re.I)

class RedisAPI(object):
    def __init__(self, factory):
        self._factory = factory
        self._connected = factory.deferred

    def __disconnected(self, *args, **kwargs):
        deferred = defer.Deferred()
        deferred.errback(RuntimeWarning("not connected"))
        return deferred

    def __getattr__(self, method):
        try:
            return getattr(self._factory.connection, method)
        except:
            return self.__disconnected

    def __wait_pool_cleanup(self, deferred):
        if self._factory.size == 0:
            self.__task.stop()
            deferred.callback(True)

    def disconnect(self):
        self._factory.continueTrying = 0
        for conn in self._factory.pool:
            try:
                conn.transport.loseConnection()
            except:
                pass

        d = defer.Deferred()
        self.__task = task.LoopingCall(self.__wait_pool_cleanup, d)
        self.__task.start(1)
        return d

    def __repr__(self):
        try:
            cli = self._factory.pool[0].transport.getPeer()
        except:
            info = "not connected"
            return info
        else:
            info = "%s:%s - %d connection(s)" % (cli.host, cli.port, self._factory.size)
            return info

class RedisShardingAPI(object):
    def __init__(self, connections):
        if isinstance(connections, defer.DeferredList):
            connections.addCallback(self.__makering)
        else:
            self.__ring = HashRing(connections)

    @defer.inlineCallbacks
    def disconnect(self):
        for conn in self.__ring.nodes:
            yield conn.disconnect()
        defer.returnValue(True)

    def __makering(self, results):
        connections = map(operator.itemgetter(1), results)
        self.__ring = HashRing(connections)
        return self

    def __wrap(self, method, *args, **kwargs):
        try:
            key = args[0]
            assert isinstance(key, types.StringTypes)
        except:
            raise ValueError("method '%s' requires a key as the first argument" % method)

        g = _findhash.match(key)

        if g != None and len(g.groups()) > 0:
            node = self.__ring(g.groups()[0])
        else:
            node = self.__ring(key)

        #print "node for '%s' is: %s" % (key, node)
        f = getattr(node, method)
        return f(*args, **kwargs)

    def __getattr__(self, method):
        if method in [
            "get", "set", "getset",
            "setnx", "setex",
            "incr", "decr", "exists",
            "delete", "get_type", "rename",
            "expire", "ttl", "push",
            "llen", "lrange", "ltrim",
            "lindex", "pop", "lset",
            "lrem", "sadd", "srem",
            "sismember", "smembers",
            "zadd", "zrem", "zincr",
            "zrange", "zrevrange", "zrangebyscore",
            "zremrangebyscore", "zcard", "zscore",
            "hget", "hset", "hdel", "hincrby", "hlen",
            "hkeys", "hvals", "hgetall", "hexists", "hmget", "hmset",
            "publish",
            ]:
            return functools.partial(self.__wrap, method)
        else:
            raise NotImplementedError("method '%s' cannot be sharded" % method)

    @defer.inlineCallbacks
    def mget(self, keys, *args):
        """
        high-level mget, required because of the sharding support
        """

        keys = protocol.list_or_args("mget", keys, args)
        group = collections.defaultdict(lambda: [])
        for k in keys:
            node = self.__ring(k)
            group[node].append(k)

        deferreds = []
        for node, keys in group.items():
            nd=node.mget(keys)
            deferreds.append(nd)

        result = []
        response = yield defer.DeferredList(deferreds)
        for (success, values) in response:
            if success:
                result += values

        defer.returnValue(result)

    def __repr__(self):
        nodes = []
        for conn in self.__ring.nodes:
            try:
                cli = conn._factory.pool[0].transport.getPeer()
            except:
                pass
            else:
                nodes.append("%s:%s/%d" % (cli.host, cli.port, conn._factory.size))
        return "<RedisSharding: %s>" % ", ".join(nodes)



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
import functools
import sys

import cyclone.redis
import cyclone.web

from twisted.python import log
from twisted.internet import defer, reactor

try:
    import txmongo
except ImportError:
    print("You need txmongo: "
          "https://github.com/fiorix/mongo-async-python-driver")
    sys.exit(1)


class Application(cyclone.web.Application):
    def __init__(self):
        mongodb = txmongo.lazyMongoConnectionPool()

        handlers = [
            (r"/", IndexHandler, dict(mongodb=mongodb)),
            (r"/createuser", CreateUserHandler, dict(mongodb=mongodb)),
        ]
        cyclone.web.Application.__init__(self, handlers, debug=True)


def HTTPBasic(method):
    @defer.inlineCallbacks
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        try:
            auth_type, auth_data = self.request.headers["Authorization"].split()
            assert auth_type == "Basic"
            usr, pwd = base64.b64decode(auth_data).split(":", 1)
        except:
            raise cyclone.web.HTTPAuthenticationRequired

        try:
            # search for user under the "cyclonedb.users" collection
            response = yield self.mongodb.cyclonedb.users.find_one(
                                        {"usr":usr, "pwd":pwd}, fields=["usr"])
            mongo_usr = response.get("usr")
        except Exception, e:
            log.msg("MongoDB failed to find(): %s" % str(e))
            raise cyclone.web.HTTPError(503) # Service Unavailable

        if usr != mongo_usr:
            raise cyclone.web.HTTPAuthenticationRequired
        else:
            self._current_user = usr
            defer.returnValue(method(self, *args, **kwargs))
    return wrapper


class IndexHandler(cyclone.web.RequestHandler):
    def initialize(self, mongodb):
        self.mongodb = mongodb

    @HTTPBasic
    def get(self):
        self.write("Hi, %s." % self._current_user)


class CreateUserHandler(cyclone.web.RequestHandler):
    def initialize(self, mongodb):
        self.mongodb = mongodb

    @defer.inlineCallbacks
    def post(self):
        usr = self.get_argument("username")
        pwd = self.get_argument("password")

        try:
            # create user under the "cyclonedb.users" collection
            ObjId = yield self.mongodb.cyclonedb.users.update(
                    {"usr":usr}, {"usr":usr, "pwd":pwd}, upsert=True, safe=True)
        except Exception, e:
            log.msg("MongoDB failed to upsert(): %s" % str(e))
            raise cyclone.web.HTTPError(503) # Service Unavailable

        self.write("User created. ObjId=%s\r\n" % ObjId)


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
import os.path
import cyclone.auth
import cyclone.escape
import cyclone.web
import uimodules
from twisted.python import log
from twisted.internet import reactor

class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", MainHandler),
            (r"/auth/login", AuthLoginHandler),
            (r"/auth/logout", AuthLogoutHandler),
        ]
        settings = dict(
            cookie_secret="12oETzKXQAGaYdkL5gEmGeJJFuYh7EQnp2XdTP1o/Vo=",
            login_url="/auth/login",
            template_path=os.path.join(os.path.dirname(__file__), "templates"),
            static_path=os.path.join(os.path.dirname(__file__), "static"),
            xsrf_cookies=True,
            facebook_api_key="9e2ada1b462142c4dfcc8e894ea1e37c",
            facebook_secret="32fc6114554e3c53d5952594510021e2",
            ui_modules= {"Post": PostModule},
            debug=True,
        )
        cyclone.web.Application.__init__(self, handlers, **settings)


class BaseHandler(cyclone.web.RequestHandler):
    def get_current_user(self):
        user_json = self.get_secure_cookie("user")
        if not user_json: return None
        return cyclone.escape.json_decode(user_json)


class MainHandler(BaseHandler, cyclone.auth.FacebookMixin):
    @cyclone.web.authenticated
    @cyclone.web.asynchronous
    def get(self):
        self.facebook_request(
            method="stream.get",
            callback=self._on_stream,
            session_key=self.current_user["session_key"])

    def _on_stream(self, stream):
        if stream is None:
            # Session may have expired
            self.redirect("/auth/login")
            return
        # Turn profiles into a dict mapping id => profile
        stream["profiles"] = dict((p["id"], p) for p in stream["profiles"])
        self.render("stream.html", stream=stream)


class AuthLoginHandler(BaseHandler, cyclone.auth.FacebookMixin):
    @cyclone.web.asynchronous
    def get(self):
        if self.get_argument("session", None):
            self.get_authenticated_user(self._on_auth)
            return
        self.authorize_redirect("read_stream")
    
    def _on_auth(self, user):
        if not user:
            raise cyclone.web.HTTPError(500, "Facebook auth failed")
        self.set_secure_cookie("user", cyclone.escape.json_encode(user))
        self.redirect(self.get_argument("next", "/"))


class AuthLogoutHandler(BaseHandler, cyclone.auth.FacebookMixin):
    @cyclone.web.asynchronous
    def get(self):
        self.clear_cookie("user")
        if not self.current_user:
            self.redirect(self.get_argument("next", "/"))
            return
        self.facebook_request(
            method="auth.revokeAuthorization",
            callback=self._on_deauthorize,
            session_key=self.current_user["session_key"])

    def _on_deauthorize(self, response):
        self.redirect(self.get_argument("next", "/"))


class PostModule(cyclone.web.UIModule):
    def render(self, post, actor):
        return self.render_string("modules/post.html", post=post, actor=actor)


def main():
    reactor.listenTCP(8888, Application())
    reactor.run()


if __name__ == "__main__":
    log.startLogging(sys.stdout)
    main()



""" 
@file protocol.py
@author Dorian Raymer
@author Ludovico Magnocavallo
@date 9/30/09
@brief Twisted compatible version of redis.py

@mainpage

txRedis is an asynchronous, Twisted, version of redis.py (included in the
redis server source).

The official Redis Command Reference:
http://code.google.com/p/redis/wiki/CommandReference

@section An example demonstrating how to use the client in your code:
@code
from twisted.internet import reactor
from twisted.internet import protocol
from twisted.internet import defer

from txredis.protocol import Redis

@defer.inlineCallbacks
def main():
    clientCreator = protocol.ClientCreator(reactor, Redis)
    redis = yield clientCreator.connectTCP(HOST, PORT)
    
    res = yield redis.ping()
    print res

    res = yield redis.set('test', 42)
    print res
    
    test = yield redis.get('test')
    print res

@endcode

Redis google code project: http://code.google.com/p/redis/
"""


import types
import warnings
from itertools import imap

from twisted.python import log
from twisted.internet import defer
from twisted.protocols import basic
from twisted.protocols import policies


class RedisError(Exception): pass
class ConnectionError(RedisError): pass
class ResponseError(RedisError): pass
class InvalidResponse(RedisError): pass
class InvalidData(RedisError): pass

def list_or_args(command, keys, args):
    oldapi = bool(args)
    try:
        i = iter(keys)
        if isinstance(keys, types.StringTypes):
            raise TypeError
    except TypeError:
        oldapi = True
        keys = [keys]

    if oldapi:
        warnings.warn(DeprecationWarning(
            "Passing *args to redis.%s is deprecated. "
            "Pass an iterable to ``keys`` instead" % command))
        keys.extend(args)
    return keys


class RedisProtocol(basic.LineReceiver, policies.TimeoutMixin):
    """The main Redis client.
    """

    ERROR = "-"
    STATUS = "+"
    INTEGER = ":"
    BULK = "$"
    MULTI_BULK = "*"

    def __init__(self, charset="utf-8", errors="strict"):
        self.charset = charset
        self.errors = errors

        self.bulk_length = 0
        self.bulk_buffer = ""
        self.multi_bulk_length = 0
        self.multi_bulk_reply = []
        self.replyQueue = defer.DeferredQueue()
        
    @defer.inlineCallbacks
    def connectionMade(self):
        if self.factory.db:
            try:
                r = yield self.select(self.factory.db)
                if isinstance(r, ResponseError):
                    raise r
            except Exception, e:
                self.factory.continueTrying = False
                self.transport.loseConnection()
                msg = "REDIS ERROR: Could not set DB=%s: %s" % (self.factory.db, e)
                self.factory.error(msg)
                if self.factory.isLazy:
                    log.msg(msg)
                defer.returnValue(None)

        self.connected = 1
        self.factory.append(self)

    def connectionLost(self, reason):
        self.connected = 0
        self.factory.remove(self)
        basic.LineReceiver.connectionLost(self, reason)

    def lineReceived(self, line):
        """
        Reply types:
          "-" error message
          "+" single line status reply
          ":" integer number (protocol level only?)
          "$" bulk data
          "*" multi-bulk data
        """
        if not line:
            return

        self.resetTimeout()
        token = line[0] # first byte indicates reply type
        data = line[1:]
        if token == self.ERROR:
            self.errorReceived(data)
        elif token == self.STATUS:
            self.statusReceived(data)
        elif token == self.INTEGER:
            self.integerReceived(data)
        elif token == self.BULK:
            try:
                self.bulk_length = long(data)
            except ValueError:
                self.replyReceived(InvalidResponse("Cannot convert data '%s' to integer" % data))
                return 
            if self.bulk_length == -1:
                self.bulkDataReceived(None)
                return
            else:
                self.bulk_length += 2 # \r\n
                self.setRawMode()
        elif token == self.MULTI_BULK:
            try:
                self.multi_bulk_length = long(data)
            except (TypeError, ValueError):
                self.replyReceived(InvalidResponse("Cannot convert multi-response header '%s' to integer" % data))
                self.multi_bulk_length = 0
                return
            if self.multi_bulk_length == -1:
                self.multi_bulk_reply = None
                self.multiBulkDataReceived()
                return
            elif self.multi_bulk_length == 0:
                self.multiBulkDataReceived()

    def rawDataReceived(self, data):
        """
        Process and dispatch to bulkDataReceived.
        """
        if self.bulk_length:
            data, rest = data[:self.bulk_length], data[self.bulk_length:]
            self.bulk_length -= len(data)
        else:
            rest = ""

        self.bulk_buffer += data
        if self.bulk_length == 0:
            buffer = self.bulk_buffer[:-2]
            self.bulk_buffer = ""
            self.bulkDataReceived(buffer)
            self.setLineMode(extra=rest)

    def errorReceived(self, data):
        """
        Error from server.
        """
        reply = ResponseError(data[4:] if data[:4] == "ERR " else data)
        self.replyReceived(reply)

    def statusReceived(self, data):
        """
        Single line status should always be a string.
        """
        if data == "none":
            reply = None # should this happen here in the client?
        else:
            reply = data 
        self.replyReceived(reply)

    def integerReceived(self, data):
        """
        For handling integer replies.
        """
        try:
            reply = int(data) 
        except ValueError:
            reply = InvalidResponse("Cannot convert data '%s' to integer" % data)
        self.replyReceived(reply)

    def bulkDataReceived(self, data):
        """
        Receipt of a bulk data element.
        """
        if data is None:
            element = data
        else:
            try:
                element = int(data) if data.find('.') == -1 else float(data)
            except (ValueError):
                try:
                    element = data.decode(self.charset)
                except UnicodeDecodeError:
                    element = data
        if self.multi_bulk_length > 0:
            self.handleMultiBulkElement(element)
        else:
            self.replyReceived(element)

    def handleMultiBulkElement(self, element):
        self.multi_bulk_reply.append(element)
        self.multi_bulk_length = self.multi_bulk_length - 1
        if self.multi_bulk_length == 0:
            self.multiBulkDataReceived()

    def multiBulkDataReceived(self):
        """
        Receipt of list or set of bulk data elements.
        """
        reply = self.multi_bulk_reply
        self.multi_bulk_reply = []
        self.multi_bulk_length = 0
        self.replyReceived(reply)

    def replyReceived(self, reply):
        """
        Complete reply received and ready to be pushed to the requesting
        function.
        """
        self.replyQueue.put(reply)


    def get_response(self):
        """return deferred which will fire with response from server.
        """
        return self.replyQueue.get()

    def encode(self, s):
        if isinstance(s, types.StringType):
            return s
        if isinstance(s, types.UnicodeType):
            try:
                return s.encode(self.charset, self.errors)
            except UnicodeEncodeError, e:
                raise InvalidData("Error encoding unicode value '%s': %s" % (repr(s), e))
        return str(s)

    def execute_command(self, *args):
        cmds = ["$%s\r\n%s\r\n" % (len(enc_value), enc_value)
            for enc_value in imap(self.encode, args)]
        self.transport.write("*%s\r\n%s" % (len(cmds), "".join(cmds)))
        return self.get_response()
    

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
    # REDIS COMMANDS
    # 

    # Connection handling
    def quit(self):
        """
        close the connection
        """
        self.factory.continueTrying = False
        return self.execute_command("QUIT")

    def auth(self, password):
        """
        simple password authentication if enabled
        """
        return self.execute_command("AUTH", password)

    def ping(self):
        """
        ping the server
        """
        return self.execute_command("PING")

    # Commands operating on all value types
    def exists(self, key):
        """
        test if a key exists
        """
        return self.execute_command("EXISTS", key)

    def delete(self, *keys):
        """
        delete one or more keys
        """
        return self.execute_command("DEL", *keys)

    def type(self, key):
        """
        return the type of the value stored at key
        """
        return self.execute_command("TYPE", key)

    def keys(self, pattern="*"):
        """
        return all the keys matching a given pattern
        """
        return self.execute_command("KEYS", pattern)

    def randomkey(self):
        """
        return a random key from the key space
        """
        return self.execute_command("RANDOMKEY")

    def rename(self, oldkey, newkey):
        """
        rename the old key in the new one, destroying the newname key if it already exists
        """
        return self.execute_command("RENAME", oldkey, newkey)

    def renamenx(self, oldkey, newkey):
        """
        rename the oldname key to newname, if the newname key does not already exist
        """
        return self.execute_command("RENAMENX", oldkey, newkey)

    def dbsize(self):
        """
        return the number of keys in the current db
        """
        return self.execute_command("DBSIZE")

    def expire(self, key, time):
        """
        set a time to live in seconds on a key
        """
        return self.execute_command("EXPIRE", key, time)

    def persist(self, key):
        """
        remove the expire from a key
        """
        return self.execute_command("PERSIST", key)

    def ttl(self, key):
        """
        get the time to live in seconds of a key
        """
        return self.execute_command("TTL", key)

    def select(self, index):
        """
        Select the DB with the specified index
        """
        return self.execute_command("SELECT", index)

    def move(self, key, dbindex):
        """
        Move the key from the currently selected DB to the dbindex DB
        """
        return self.execute_command("MOVE", key, dbindex)

    def flush(self, all_dbs=False):
        warnings.warn(DeprecationWarning(
            "redis.flush() has been deprecated, "
            "use redis.flush() or redis.flushall() instead"))
        return all_dbs and self.flushall() or self.flushdb()

    def flushdb(self):
        """
        Remove all the keys from the currently selected DB
        """
        return self.execute_command("FLUSHDB")

    def flushall(self):
        """
        Remove all the keys from all the databases
        """
        return self.execute_command("FLUSHALL")

    # Commands operating on string values
    def set(self, key, value, preserve=False, getset=False):
        """
        Set a key to a string value
        """
        if preserve:
            warnings.warn(DeprecationWarning(
                "preserve option to 'set' is deprecated, "
                "use redis.setnx() instead"))
            return self.setnx(key, value)

        if getset:
            warnings.warn(DeprecationWarning(
                "getset option to 'set' is deprecated, "
                "use redis.getset() instead"))
            return self.getset(key, value)

        return self.execute_command("SET", key, value)
    
    def get(self, key):
        """
        Return the string value of the key
        """
        return self.execute_command("GET", key)
    
    def getset(self, key, value):
        """
        Set a key to a string returning the old value of the key
        """
        return self.execute_command("GETSET", key, value)
        
    def mget(self, keys, *args):
        """
        Multi-get, return the strings values of the keys
        """
        keys = list_or_args("mget", keys, args)
        return self.execute_command("MGET", *keys)
    
    def setnx(self, key, value):
        """
        Set a key to a string value if the key does not exist
        """
        return self.execute_command("SETNX", key, value)

    def setex(self, key, time, value):
        """
        Set+Expire combo command
        """
        return self.execute_command("SETEX", key, time, value)

    def mset(self, mapping):
        """
        Set the respective fields to the respective values. HMSET replaces old values with new values.
        """
        items = []
        for pair in mapping.iteritems():
            items.extend(pair)
        return self.execute_command("MSET", *items)

    def msetnx(self, mapping):
        """
        Set multiple keys to multiple values in a single atomic operation if none of the keys already exist
        """
        items = []
        for pair in mapping.iteritems():
            items.extend(pair)
        return self.execute_command("MSETNX", *items)

    def incr(self, key, amount=1):
        """
        Increment the integer value of key
        """
        return self.execute_command("INCRBY", key, amount)

    def incrby(self, key, amount):
        """
        Increment the integer value of key by integer
        """
        return self.incr(key, amount)

    def decr(self, key, amount=1):
        """
        Decrement the integer value of key
        """
        return self.execute_command("DECRBY", key, amount)

    def decrby(self, key, amount):
        """
        Decrement the integer value of key by integer
        """
        return self.decr(key, amount)

    def append(self, key, value):
        """
        Append the specified string to the string stored at key
        """
        return self.execute_command("APPEND", key, value)

    def substr(self, key, start, end=-1):
        """
        Return a substring of a larger string
        """
        return self.execute_command("SUBSTR", key, start, end)

    # Commands operating on lists
    def push(self, key, value, tail=False):
        warnings.warn(DeprecationWarning(
            "redis.push() has been deprecated, "
            "use redis.lpush() or redis.rpush() instead"))

        return tail and self.rpush(key, value) or self.lpush(key, value)

    def rpush(self, key, value):
        """
        Append an element to the tail of the List value at key
        """
        return self.execute_command("RPUSH", key, value)

    def lpush(self, key, value):
        """
        Append an element to the head of the List value at key
        """
        return self.execute_command("LPUSH", key, value)
    
    def llen(self, key):
        """
        Return the length of the List value at key
        """
        return self.execute_command("LLEN", key)

    def lrange(self, key, start, end):
        """
        Return a range of elements from the List at key
        """
        return self.execute_command("LRANGE", key, start, end)
        
    def ltrim(self, key, start, end):
        """
        Trim the list at key to the specified range of elements
        """
        return self.execute_command("LTRIM", key, start, end)
    
    def lindex(self, key, index):
        """
        Return the element at index position from the List at key
        """
        return self.execute_command("LINDEX", key, index)

    def lset(self, key, index, value):
        """
        Set a new value as the element at index position of the List at key
        """
        return self.execute_command("LSET", key, index, value)

    def lrem(self, key, count, value):
        """
        Remove the first-N, last-N, or all the elements matching value from the List at key
        """
        return self.execute_command("LREM", key, count, value)
        
    def pop(self, key, tail=False):
        warnings.warn(DeprecationWarning(
            "redis.pop() has been deprecated, "
            "user redis.lpop() or redis.rpop() instead"))

        return tail and self.rpop(key) or self.lpop(key)

    def lpop(self, key):
        """
        Return and remove (atomically) the first element of the List at key
        """
        return self.execute_command("LPOP", key)

    def rpop(self, key):
        """
        Return and remove (atomically) the last element of the List at key
        """
        return self.execute_command("RPOP", key)

    def blpop(self, keys, timeout=0):
        """
        Blocking LPOP
        """
        if isinstance(keys, types.StringTypes):
            keys = [keys]
        else:
            keys = list(keys)

        keys.append(timeout)
        return self.execute_command("BLPOP", *keys)

    def brpop(self, keys, timeout=0):
        """
        Blocking RPOP
        """
        if isinstance(keys, types.StringTypes):
            keys = [keys]
        else:
            keys = list(keys)

        keys.append(timeout)
        return self.execute_command("BRPOP", *keys)
    
    def rpoplpush(self, srckey, dstkey):
        """
        Return and remove (atomically) the last element of the source 
        List  stored at srckey and push the same element to the 
        destination List stored at dstkey
        """
        return self.execute_command("RPOPLPUSH", srckey, dstkey)
    
    # Commands operating on sets
    def sadd(self, key, member):
        """
        Add the specified member to the Set value at key
        """
        return self.execute_command("SADD", key, member)
        
    def srem(self, key, member):
        """
        Remove the specified member from the Set value at key
        """
        return self.execute_command("SREM", key, member)

    def spop(self, key):
        """
        Remove and return (pop) a random element from the Set value at key
        """
        return self.execute_command("SPOP", key)

    def smove(self, srckey, dstkey, member):
        """
        Move the specified member from one Set to another atomically
        """
        return self.execute_command("SMOVE", srckey, dstkey, member)

    def scard(self, key):
        """
        Return the number of elements (the cardinality) of the Set at key
        """
        return self.execute_command("SCARD", key)
    
    def sismember(self, key, value):
        """
        Test if the specified value is a member of the Set at key
        """
        return self.execute_command("SISMEMBER", key, value)
    
    def sinter(self, keys, *args):
        """
        Return the intersection between the Sets stored at key1, key2, ..., keyN
        """
        keys = list_or_args("sinter", keys, args)
        return self.execute_command("SINTER", *keys)
    
    def sinterstore(self, dstkey, keys, *args):
        """
        Compute the intersection between the Sets stored 
        at key1, key2, ..., keyN, and store the resulting Set at dstkey
        """
        keys = list_or_args("sinterstore", keys, args)
        return self.execute_command("SINTERSTORE", dstkey, *keys)

    def sunion(self, keys, *args):
        """
        Return the union between the Sets stored at key1, key2, ..., keyN
        """
        keys = list_or_args("sunion", keys, args)
        return self.execute_command("SUNION", *keys)

    def sunionstore(self, dstkey, keys, *args):
        """
        Compute the union between the Sets stored 
        at key1, key2, ..., keyN, and store the resulting Set at dstkey
        """
        keys = list_or_args("sunionstore", keys, args)
        return self.execute_command("SUNIONSTORE", dstkey, *keys)

    def sdiff(self, keys, *args):
        """
        Return the difference between the Set stored at key1 and all the Sets key2, ..., keyN
        """
        keys = list_or_args("sdiff", keys, args)
        return self.execute_command("SDIFF", *keys)

    def sdiffstore(self, dstkey, keys, *args):
        """
        Compute the difference between the Set key1 and all the 
        Sets key2, ..., keyN, and store the resulting Set at dstkey
        """
        keys = list_or_args("sdiffstore", keys, args)
        return self.execute_command("SDIFFSTORE", dstkey, *keys)

    def smembers(self, key):
        """
        Return all the members of the Set value at key
        """
        return self.execute_command("SMEMBERS", key)

    def srandmember(self, key):
        """
        Return a random member of the Set value at key
        """
        return self.execute_command("SRANDMEMBER", key)

    # Commands operating on sorted zsets (sorted sets)
    def zadd(self, key, score, member):
        """
        Add the specified member to the Sorted Set value at key 
        or update the score if it already exist
        """
        return self.execute_command("ZADD", key, score, member)
        
    def zrem(self, key, member):
        """
        Remove the specified member from the Sorted Set value at key
        """
        return self.execute_command("ZREM", key, member)
    
    def zincr(self, key, member):
        return self.zincrby(key, 1, member)

    def zdecr(self, key, member):
        return self.zincrby(key, -1, member)

    def zincrby(self, key, increment, member):
        """
        If the member already exists increment its score by increment, 
        otherwise add the member setting increment as score
        """
        return self.execute_command("ZINCRBY", key, increment, member)

    def zrank(self, key, member):
        """
        Return the rank (or index) or member in the sorted set at key, 
        with scores being ordered from low to high
        """
        return self.execute_command("ZRANK", key, member)

    def zrevrank(self, key, member):
        """
        Return the rank (or index) or member in the sorted set at key, 
        with scores being ordered from high to low
        """
        return self.execute_command("ZREVRANK", key, member)

    def zrange(self, key, start, end):
        """
        Return a range of elements from the sorted set at key

        """
        return self.execute_command("ZRANGE", key, start, end)
   
    def zrevrange(self, key, start, end):
        """
        Return a range of elements from the sorted set at key, 
        exactly like ZRANGE, but the sorted set is ordered in 
        traversed in reverse order, from the greatest to the smallest score
        """
        return self.execute_command("ZREVRANGE", key, start, end)
    
    def zrangebyscore(self, key, min, max):
        """
        Return all the elements with score >= min and score <= max (a range query) from the sorted set
        """
        return self.execute_command("ZRANGEBYSCORE", key, min, max)

    def zcount(self, key, min, max):
        """
        Return the number of elements with score >= min and score <= max in the sorted set
        """
        return self.execute_command("ZCOUNT", key, min, max)

    def zcard(self, key):
        """
        Return the cardinality (number of elements) of the sorted set at key
        """
        return self.execute_command("ZCARD", key)

    def zscore(self, key, element):
        """
        Return the score associated with the specified element of the sorted set at key
        """
        return self.execute_command("ZSCORE", key, element)

    def zremrangebyrank(self, key, min, max):
        """
        Remove all the elements with rank >= min and rank <= max from the sorted set
        """
        return self.execute_command("ZREMRANGEBYRANK", key, min, max)

    def zremrangebyscore(self, key, min, max):
        """
        Remove all the elements with score >= min and score <= max from the sorted set
        """
        return self.execute_command("ZREMRANGEBYSCORE", key, min, max)

    def zunionstore(self, dstkey, keys, aggregate=None):
        """
        Perform a union over a number of sorted sets with optional weight and aggregate
        """
        return self._zaggregate("ZUNIONSTORE", dstkey, keys, aggregate)

    def zinterstore(self, dstkey, keys, aggregate=None):
        """
        Perform an intersection over a number of sorted sets with optional weight and aggregate
        """
        return self._zaggregate("ZINTERSTORE", dstkey, keys, aggregate)

    def _zaggregate(self, command, dstkey, keys, aggregate):
        pieces = [command, dstkey, len(keys)]
        if isinstance(keys, types.DictType):
            items = keys.items()
            keys = [i[0] for i in items]
            weights = [i[1] for i in items]
        else:
            weights = None

        pieces.extend(keys)
        if weights:
            pieces.append("WEIGHTS")
            pieces.extend(weights)
        if aggregate:
            pieces.append("AGGREGATE")
            pieces.append(aggregate)

        return self.execute_command(*pieces)

    # Commands operating on hashes
    def hset(self, key, field, value):
        """
        Set the hash field to the specified value. Creates the hash if needed
        """
        return self.execute_command("HSET", key, field, value)

    def hsetnx(self, key, field, value):
        """
        Set the hash field to the specified value if the field does not exist. 
        Creates the hash if needed
        """
        return self.execute_command("HSETNX", key, field, value)
    
    def hget(self, key, field):
        """
        Retrieve the value of the specified hash field.
        """
        return self.execute_command("HGET", key, field)

    def hmget(self, key, fields):
        """
        Get the hash values associated to the specified fields.
        """
        return self.execute_command("HMGET", key, *fields)

    def hmset(self, key, mapping):
        """
        Set the hash fields to their respective values.
        """
        items = []
        for pair in mapping.iteritems():
            items.extend(pair)
        return self.execute_command("HMSET", key, *items)

    def hincr(self, key, field):
        return self.hincrby(key, field, 1)

    def hdecr(self, key, field):
        return self.hincrby(key, field, -1)
        
    def hincrby(self, key, field, integer):
        """
        Increment the integer value of the hash at key on field with integer.
        """
        return self.execute_command("HINCRBY", key, field, integer)

    def hexists(self, key, field):
        """
        Test for existence of a specified field in a hash
        """
        return self.execute_command("HEXISTS", key, field)

    def hdel(self, key, field):
        """
        Remove the specified field from a hash
        """
        return self.execute_command("HDEL", key, field)
    
    def hlen(self, key):
        """
        Return the number of items in a hash.
        """
        return self.execute_command("HLEN", key)
    
    def hkeys(self, key):
        """
        Return all the fields in a hash.
        """
        return self.execute_command("HKEYS", key)
    
    def hvals(self, key):
        """
        Return all the values in a hash.
        """
        return self.execute_command("HVALS", key)
    
    @defer.inlineCallbacks
    def hgetall(self, key):
        """
        Return all the fields and associated values in a hash.
        """
        d = yield self.execute_command("HGETALL", key)
        defer.returnValue(dict(zip(d[::2], d[1::2])))

    # Sorting
    def sort(self, key, start=None, end=None, by=None, get=None,
            desc=None, alpha=False, store=None):
        if (start is not None and end is not None) or \
            (end is not None and start is None):
            raise RedisError("``start`` and ``end`` must both be specified")

        pieces = [key]
        if by is not None:
            pieces.append("BY")
            pieces.append(by)
        if start is not None and end is not None:
            pieces.append("LIMIT")
            pieces.append(start)
            pieces.append(end)
        if get is not None:
            pieces.append("GET")
            pieces.append(get)
        if desc:
            pieces.append("DESC")
        if alpha:
            pieces.append("ALPHA")
        if store is not None:
            pieces.append("STORE")
            pieces.append(store)

        return self.execute_command("SORT", *pieces)

    # Transactions
    # TODO

    # Publish/Subscribe
    # see the SubscriberProtocol for subscribing to channels
    def publish(self, channel, message):
        """
        Publish message to a channel 
        """
        return self.execute_command("PUBLISH", channel, message)

    # Persistence control commands
    def save(self):
        """
        Synchronously save the DB on disk
        """
        return self.execute_command("SAVE")

    def bgsave(self):
        """
        Asynchronously save the DB on disk
        """
        return self.execute_command("BGSAVE")

    def lastsave(self):
        """
        Return the UNIX time stamp of the last successfully saving of the dataset on disk
        """
        return self.execute_command("LASTSAVE")

    def shutdown(self):
        """
        Synchronously save the DB on disk, then shutdown the server
        """
        self.factory.continueTrying = False
        return self.execute_command("SHUTDOWN")

    def bgrewriteaof(self):
        """
        Rewrite the append only file in background when it gets too big
        """
        return self.execute_command("BGREWRITEAOF")

    # Remote server control commands
    def info(self):
        """
        Provide information and statistics about the server
        """
        return self.execute_command("INFO")

    # monitor and slaveof should are missing


class SubscriberProtocol(RedisProtocol):
    def connectionLost(self, reason):
        pass

    def messageReceived(self, pattern, channel, message):
        pass

    def replyReceived(self, reply):
        if type(reply) is types.ListType:
            if reply[-3] == u"message":
                self.messageReceived(None, *reply[-2:])
            else:
                self.messageReceived(*reply[-3:])

    def subscribe(self, channels):
        if isinstance(channels, types.StringTypes):
            channels = [channels]
        return self.execute_command("SUBSCRIBE", *channels)

    def unsubscribe(self, channels):
        if isinstance(channels, types.StringTypes):
            channels = [channels]
        return self.execute_command("UNSUBSCRIBE", *channels)

    def psubscribe(self, patterns):
        if isinstance(patterns, types.StringTypes):
            patterns = [patterns]
        return self.execute_command("PSUBSCRIBE", *patterns)

    def punsubscribe(self, patterns):
        if isinstance(patterns, types.StringTypes):
            patterns = [patterns]
        return self.execute_command("PUNSUBSCRIBE", *patterns)



#!/usr/bin/env python

import cyclone.escape
import cyclone.web
import cyclone.websocket
import os.path
import uuid, sys
from twisted.python import log
from twisted.internet import reactor

class Application(cyclone.web.Application):
    def __init__(self):
        settings = dict(
            cookie_secret="43oETzKXQAGaYdkL5gEmGeJJFuYh7EQnp2XdTP1o/Vo=",
            template_path=os.path.join(os.path.dirname(__file__), "templates"),
            static_path=os.path.join(os.path.dirname(__file__), "static"),
            xsrf_cookies=True,
            autoescape=None,
        )
        
        handlers = [
            (r"/", MainHandler),
            (r"/echo", EchoSocketHandler),
            (r"/(jquery-latest\.js)", cyclone.web.StaticFileHandler,
                dict(path=settings['static_path'])),
        ]
        cyclone.web.Application.__init__(self, handlers, **settings)

class MainHandler(cyclone.web.RequestHandler):
    def get(self):
        self.render("echo.html")

class EchoSocketHandler(cyclone.websocket.WebSocketHandler):

    def connectionMade(self, *args, **kwargs):
        log.msg("ws opened")

    def connectionLost(self, reason):
        log.msg("ws closed")

    def messageReceived(self, message):
        log.msg("got message %s" % message)
        self.sendMessage(message)

def main():
    reactor.listenTCP(8888, Application())
    reactor.run()


if __name__ == "__main__":
    log.startLogging(sys.stdout)
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import cyclone.escape
import cyclone.redis
import cyclone.sqlite
import cyclone.util
import cyclone.web
import cyclone.websocket
import cyclone.xmlrpc
from cyclone.bottle import run, route

from twisted.internet import defer

class BaseHandler(cyclone.web.RequestHandler):
    @property
    def redisdb(self):
        return self.settings.db_handlers.redis

    def get_current_user(self):
        print "Getting user cookie"
        return self.get_secure_cookie("user")


@route("/")
def index(cli):
    cli.write('<a href="/auth/login">sign in</a>')


@route("/auth/login")
def auth_login_page(cli):
    cli.write("""
    <html><body><form method="post">
    username: <input type="text" name="usr"/><br/>
    password: <input type="password" name="pwd"/><br/>
    <input type="submit">
    </form></body></html>
    """)


@route("/auth/login", method="post")
@defer.inlineCallbacks
def auth_login(cli):
    usr = cli.get_argument("usr")
    pwd = cli.get_argument("pwd")

    try:
        redis_pwd = yield cli.redisdb.get("cyclone:%s" % usr)
    except Exception, e:
        log.msg("Redis failed to get('cyclone:%s'): %s" % (usr, str(e)))
        raise cyclone.web.HTTPError(503) # Service Unavailable

    if pwd != str(redis_pwd):
        cli.write("Invalid user or password.<br>"
                  '<a href="/auth/login">try again</a>')
    else:
        cli.set_secure_cookie("user", usr)
        cli.redirect(cli.get_argument("next", "/private"))


@route("/auth/logout")
@cyclone.web.authenticated
def auth_logout(cli):
    cli.clear_cookie("user")
    cli.redirect("/")


@route("/private")
@cyclone.web.authenticated
def private(cli):
    cli.write("Hi, %s<br/>" % cli.current_user)
    cli.write("""<a href="/auth/logout">logout</a>""")


class WebSocketHandler(cyclone.websocket.WebSocketHandler):
    def connectionMade(self, *args, **kwargs):
        print "connection made:", args, kwargs

    def messageReceived(self, message):
        self.sendMessage("echo: %s" % message)

    def connectionLost(self, why):
        print "connection lost:", why


class XmlrpcHandler(cyclone.xmlrpc.XmlrpcRequestHandler):
    allowNone = True
    def xmlrpc_echo(self, text):
        return text


try:
    raise COMMENT_THIS_LINE_AND_LOG_TO_DAILY_FILE
    from twisted.python.logfile import DailyLogFile
    logFile = DailyLogFile.fromFullPath("server.log")
    print("Logging to daily log file: server.log")
except Exception, e:
    import sys
    logFile = sys.stdout

run(host="127.0.0.1", port=8888,
    log=logFile,
    debug=True,
    static_path="./static",
    template_path="./template",
    locale_path="./locale",
    login_url="/auth/login",
    cookie_secret="32oETzKXQAGaYdkL5gEmGeJJFuYh7EQnp2XdTP1o/Vo=",
    base_handler=BaseHandler,
    db_handlers=cyclone.util.ObjectDict(
        #sqlite=cyclone.sqlite.InlineSQLite(":memory:"),
        redis=cyclone.redis.lazyRedisConnectionPool(),
    ),
    more_handlers=[
        (r"/websocket", WebSocketHandler),
        (r"/xmlrpc",    XmlrpcHandler),
    ])




#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import cyclone.httpclient
from twisted.internet import defer, reactor

@defer.inlineCallbacks
def main():
    cli = cyclone.httpclient.JsonRPC("http://localhost:8888/jsonrpc")
    print "echo:", (yield cli.echo("foo bar"))
    print "sort:", (yield cli.sort(["foo", "bar"]))
    print "count:", (yield cli.count(["foo", "bar"]))
    print "geoip_lookup:", (yield cli.geoip_lookup("google.com"))
    reactor.stop()

if __name__ == "__main__":
    main()
    reactor.run()



#
# consistent hashing for redis client
# based on ezmobius client (redis-rb)
#
import zlib
import bisect

class HashRing(object):
    """Consistent hash for redis API"""
    def __init__(self, nodes=[], replicas=160):
        self.nodes=[]
        self.replicas=replicas
        self.ring={}
        self.sorted_keys=[]

        for n in nodes:
            self.add_node(n)

    def add_node(self, node):
        self.nodes.append(node)
        for x in xrange(self.replicas):
            crckey = zlib.crc32("%s:%d" % (node, x))
            self.ring[crckey] = node
            self.sorted_keys.append(crckey)
        
        self.sorted_keys.sort() 

    def remove_node(self, node):
        self.nodes.remove(node)
        for x in xrange(self.replicas):
            crckey = zlib.crc32("%s:%d" % (node, x))
            self.ring.remove(crckey)
            self.sorted_keys.remove(crckey)
        
    def get_node(self, key):
        n, i = self.get_node_pos(key)
        return n
    #self.get_node_pos(key)[0]

    def get_node_pos(self, key):
        if len(self.ring) == 0:
            return [None, None]
        crc = zlib.crc32(key)
        idx = bisect.bisect(self.sorted_keys, crc)
        idx = min(idx, (self.replicas * len(self.nodes))-1) # prevents out of range index
        return [self.ring[self.sorted_keys[idx]], idx]

    def iter_nodes(self, key):
        if len(self.ring) == 0: yield None, None
        node, pos = self.get_node_pos(key)
        for k in self.sorted_keys[pos:]:
            yield k, self.ring[k]
    
    def __call__(self, key):
        return self.get_node(key)



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import sys

import cyclone.web

from twisted.python import log
from twisted.internet import reactor


# Helper function to convert bytes to human readable strings
humanreadable = lambda s:[(s%1024**i and "%.1f"%(s/1024.0**i) or \
                          str(s/1024**i))+x.strip()+"B" \
                          for i,x in enumerate(' KMGTPEZY') \
                          if s<1024**(i+1) or i==8][0]


class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", IndexHandler),
        ]
        settings = dict(
            debug=True,
            template_path="./template",
            repository_path="./uploaded_files",
        )

        if not os.path.exists(settings["repository_path"]):
            try:
                os.mkdir(settings["repository_path"])
            except Exception, e:
                print("mkdir failed: %s" % str(e))
                sys.exit(1)

        cyclone.web.Application.__init__(self, handlers, **settings)


class IndexHandler(cyclone.web.RequestHandler):
    def get(self):
        self.render("index.html", missing=[], info=None)

    def post(self):
        name = self.get_argument("fullname", None)
        if name is None:
            self.render("index.html", missing=["fullname"], info=None)
            return

        picture = self.request.files.get("picture")
        if picture is None:
            self.render("index.html", missing=["picture"], info=None)
            return
        else:
            picture = picture[0]

        # File properties
        filename = picture["filename"]
        content_type = picture["content_type"]
        body = picture["body"] # bytes!

        try:
            fn = os.path.join(self.settings.repository_path, filename)
            fp = open(os.path.abspath(fn), "w")
            fp.write(body)
            fp.close()
        except Exception, e:
            log.msg("Could not write file: %s" % str(e))
            raise cyclone.web.HTTPError(500)

        self.render("index.html", missing=[], info={
                    "name":name,
                    "file":"%s, type=%s, size=%s" % \
                    (filename, content_type, humanreadable(len(body)))})


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
import functools
import os.path
import sys

import cyclone.web
import cyclone.redis
from cyclone.redis.protocol import SubscriberProtocol

from twisted.python import log
from twisted.internet import defer, reactor


class Application(cyclone.web.Application):
    def __init__(self):
        RedisMixin().makeItFunk("127.0.0.1", 6379, 0, 10)

        handlers = [
            (r"/text/(.+)", TextHandler),
            (r"/queue/(.+)", QueueHandler),
        ]
        settings = dict(
            debug=True,
            static_path="./frontend/static",
            template_path="./frontend/template",
        )
        cyclone.web.Application.__init__(self, handlers, **settings)


class RedisMixin(object):
    dbconn = None
    psconn = None
    channels = collections.defaultdict(lambda: [])

    def makeItFunk(self, host, port, dbid, pool_size):
        cls = RedisMixin

        # PubSub client connection
        qf = cyclone.redis.SubscriberFactory()
        qf.maxDelay = 20
        qf.continueTrying = True # Auto-reconnect
        qf.protocol = QueueProtocol
        reactor.connectTCP(host, port, qf)

        # Normal client connection
        cls.dbconn = cyclone.redis.lazyRedisConnectionPool(host, port, db=dbid,
                                                           pool_size=pool_size)

    def subscribe(self, channel):
        if RedisMixin.psconn is None:
            raise cyclone.web.HTTPError(503) # Service Unavailable

        if channel not in RedisMixin.channels:
            log.msg("Subscribing entire server to %s" % channel)
            if "*" in channel:
                RedisMixin.psconn.psubscribe(channel)
            else:
                RedisMixin.psconn.subscribe(channel)

        RedisMixin.channels[channel].append(self)
        log.msg("Client %s subscribed to %s" % \
                (self.request.remote_ip, channel))

    def unsubscribe_all(self, ign):
        # Unsubscribe peer from all channels
        for channel, peers in RedisMixin.channels.iteritems():
            try:
                peers.pop(peers.index(self))
            except:
                continue

            log.msg("Client %s unsubscribed from %s" % \
                    (self.request.remote_ip, channel))

            # Unsubscribe from channel if no peers are listening
            if not len(peers) and RedisMixin.psconn is not None:
                log.msg("Unsubscribing entire server from %s" % channel)
                if "*" in channel:
                    RedisMixin.psconn.punsubscribe(channel)
                else:
                    RedisMixin.psconn.unsubscribe(channel)

    def broadcast(self, pattern, channel, message):
        peers = RedisMixin.channels.get(pattern or channel)

        # Broadcast the message to all peers in channel
        for peer in peers:
            # peer is an HTTP client, RequestHandler
            peer.write("%s: %s\r\n" % (channel, message))
            peer.flush()


# Provide GET, SET and DELETE redis operations via HTTP
class TextHandler(cyclone.web.RequestHandler, RedisMixin):
    @defer.inlineCallbacks
    def get(self, key):
        try:
            value = yield RedisMixin.dbconn.get(key)
        except Exception, e:
            log.err("Redis failed to get('%s'): %s" % (key, str(e)))
            raise cyclone.web.HTTPError(503)

        self.set_header("Content-Type", "text/plain")
        self.finish("%s=%s\r\n" % (key, value))

    @defer.inlineCallbacks
    def post(self, key):
        value = self.get_argument("value")
        try:
            yield RedisMixin.dbconn.set(key, value)
        except Exception, e:
            log.err("Redis failed to set('%s', '%s'): %s" % (key, value, str(e)))
            raise cyclone.web.HTTPError(503)

        self.set_header("Content-Type", "text/plain")
        self.finish("%s=%s\r\n" % (key, value))

    @defer.inlineCallbacks
    def delete(self, key):
        try:
            n = yield RedisMixin.dbconn.delete(key)
        except Exception, e:
            log.err("Redis failed to del('%s'): %s" % (key, str(e)))
            raise cyclone.web.HTTPError(503)

        self.set_header("Content-Type", "text/plain")
        self.finish("DEL %s=%d\r\n" % (key, n))


# GET will subscribe to channels or patterns
# POST will (obviously) post messages to channels
class QueueHandler(cyclone.web.RequestHandler, RedisMixin):
    @cyclone.web.asynchronous
    def get(self, channels):
        try:
            channels = channels.split(",")
        except Exception, e:
            log.err("Could not split channel names: %s" % str(e))
            raise cyclone.web.HTTPError(400, str(e))

        self.set_header("Content-Type", "text/plain")
        self.notifyFinish().addCallback(
            functools.partial(RedisMixin.unsubscribe_all, self))

        for channel in channels:
            RedisMixin.subscribe(self, channel)
            self.write("subscribed to %s\r\n" % channel)
        self.flush()

    @defer.inlineCallbacks
    def post(self, channel):
        message = self.get_argument("message")

        try:
            n = yield RedisMixin.dbconn.publish(channel,
                                                message.encode("utf-8"))
        except Exception, e:
            log.msg("Redis failed to publish('%s', '%s'): %s" % \
                    (channel, repr(message), str(e)))
            raise cyclone.web.HTTPError(503)

        self.set_header("Content-Type", "text/plain")
        self.finish("OK %d\r\n" % n)


class QueueProtocol(SubscriberProtocol, RedisMixin):
    def messageReceived(self, pattern, channel, message):
        # When new messages are published to Redis channels or patterns,
        # they are broadcasted to all HTTP clients subscribed to those
        # channels.
        RedisMixin.broadcast(self, pattern, channel, message)

    def connectionMade(self):
        RedisMixin.psconn = self

        # If we lost connection with Redis during operation, we
        # re-subscribe to all channels once the connection is re-established.
        for channel in RedisMixin.channels:
            if "*" in channel:
                self.psubscribe(channel)
            else:
                self.subscribe(channel)

    def connectionLost(self, why):
        RedisMixin.psconn = None


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
import cyclone.web
import cyclone.auth
import cyclone.escape
from twisted.python import log
from twisted.internet import reactor

import os.path
import uuid

class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", MainHandler),
            (r"/auth/login", AuthLoginHandler),
            (r"/auth/logout", AuthLogoutHandler),
            (r"/a/message/new", MessageNewHandler),
            (r"/a/message/updates", MessageUpdatesHandler),
        ]
        settings = dict(
            cookie_secret="43oETzKXQAGaYdkL5gEmGeJJFuYh7EQnp2XdTP1o/Vo=",
            login_url="/auth/login",
            template_path=os.path.join(os.path.dirname(__file__), "templates"),
            static_path=os.path.join(os.path.dirname(__file__), "static"),
            xsrf_cookies=True,
            autoescape=None,
        )
        cyclone.web.Application.__init__(self, handlers, **settings)


class BaseHandler(cyclone.web.RequestHandler):
    def get_current_user(self):
        user_json = self.get_secure_cookie("user")
        if not user_json: return None
        return cyclone.escape.json_decode(user_json)


class MainHandler(BaseHandler):
    @cyclone.web.authenticated
    def get(self):
        self.render("index.html", messages=MessageMixin.cache)


class MessageMixin(object):
    waiters = []
    cache = []
    cache_size = 200

    def wait_for_messages(self, callback, cursor=None):
        cls = MessageMixin
        if cursor:
            index = 0
            for i in xrange(len(cls.cache)):
                index = len(cls.cache) - i - 1
                if cls.cache[index]["id"] == cursor: break
            recent = cls.cache[index + 1:]
            if recent:
                callback(recent)
                return
        cls.waiters.append(callback)

    def new_messages(self, messages):
        cls = MessageMixin
        log.msg("Sending new message to %r listeners" % len(cls.waiters))
        for callback in cls.waiters:
            try:
                callback(messages)
            except:
                log.err()
        cls.waiters = []
        cls.cache.extend(messages)
        if len(cls.cache) > self.cache_size:
            cls.cache = cls.cache[-self.cache_size:]


class MessageNewHandler(BaseHandler, MessageMixin):
    @cyclone.web.authenticated
    def post(self):
        message = {
            "id": str(uuid.uuid4()),
            "from": self.current_user["first_name"],
            "body": self.get_argument("body"),
        }
        message["html"] = self.render_string("message.html", message=message)
        if self.get_argument("next", None):
            self.redirect(self.get_argument("next"))
        else:
            self.write(message)
        self.new_messages([message])


class MessageUpdatesHandler(BaseHandler, MessageMixin):
    @cyclone.web.authenticated
    @cyclone.web.asynchronous
    def post(self):
        cursor = self.get_argument("cursor", None)
        self.wait_for_messages(self.on_new_messages, cursor=cursor)

    def on_new_messages(self, messages):
        # Closed client connection
        #if self.request.connection.stream.closed():
            #return
        self.finish(dict(messages=messages))


class AuthLoginHandler(BaseHandler, cyclone.auth.GoogleMixin):
    @cyclone.web.asynchronous
    def get(self):
        if self.get_argument("openid.mode", None):
            self.get_authenticated_user(self._on_auth)
            return
        self.authenticate_redirect(ax_attrs=["name"])
    
    def _on_auth(self, user):
        if not user:
            raise cyclone.web.HTTPError(500, "Google auth failed")
        self.set_secure_cookie("user", cyclone.escape.json_encode(user))
        self.redirect("/")


class AuthLogoutHandler(BaseHandler):
    def get(self):
        self.clear_cookie("user")
        self.write("You are now logged out")


def main():
    reactor.listenTCP(8888, Application())
    reactor.run()


if __name__ == "__main__":
    log.startLogging(sys.stdout)
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

import cyclone.httpclient
import cyclone.jsonrpc
import cyclone.xmlrpc

from twisted.python import log
from twisted.internet import defer, reactor


class XmlrpcHandler(cyclone.xmlrpc.XmlrpcRequestHandler):
    allowNone = True

    def xmlrpc_echo(self, text):
        return text

    def xmlrpc_sort(self, items):
        return sorted(items)

    def xmlrpc_count(self, items):
        return len(items)

    @defer.inlineCallbacks
    def xmlrpc_geoip_lookup(self, address):
        result = yield cyclone.httpclient.fetch(
            "http://freegeoip.net/xml/%s" % address)
        defer.returnValue(result.body)


class JsonrpcHandler(cyclone.jsonrpc.JsonrpcRequestHandler):
    def jsonrpc_echo(self, text):
        return text

    def jsonrpc_sort(self, items):
        return sorted(items)

    def jsonrpc_count(self, items):
        return len(items)

    @defer.inlineCallbacks
    def jsonrpc_geoip_lookup(self, address):
        result = yield cyclone.httpclient.fetch(
            "http://freegeoip.net/json/%s" % address.encode("utf-8"))
        defer.returnValue(result.body)


def main():
    log.startLogging(sys.stdout)
    application = cyclone.web.Application([
        (r"/xmlrpc", XmlrpcHandler),
        (r"/jsonrpc", JsonrpcHandler),
    ])

    reactor.listenTCP(8888, application)
    reactor.run()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
import cyclone.escape
import cyclone.httpclient
from twisted.internet import defer, reactor

@defer.inlineCallbacks
def test():
    print("get http://google.com followRecirect=0")
    response = yield cyclone.httpclient.fetch("http://google.com/")
    print("headers:", response.headers)
    print("code and phrase:", response.code, response.phrase)
    print("length and body:", response.length, response.body)

    print("\nget http://google.com followRecirect=1")
    response = yield cyclone.httpclient.fetch("http://google.com/",
                                              followRedirect=1, maxRedirects=2)
    print("headers:", response.headers)
    print("code and phrase:", response.code, response.phrase)
    print("length and body:", response.length, response.body)

    #print("\npost")
    #json = cyclone.escape.json_encode({"user":"foo", "pass":"bar"})
    #response = yield cyclone.httpclient.fetch("http://localhost:8888/",
    #                        followRedirect=1, maxRedirects=2, postdata=json,
    #                        headers={"Content-Type":["application/json"]})
    #print("headers:", response.headers)
    #print("code and phrase:", response.code, response.phrase)
    #print("length and body:", response.length, response.body)


if __name__ == "__main__":
    test().addCallback(lambda ign: reactor.stop())
    reactor.run()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os.path
import sys

import cyclone.sse
import cyclone.web

from twisted.internet import defer, protocol, reactor
from twisted.protocols import telnet
from twisted.python import log

class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", cyclone.web.RedirectHandler, {"url": "/static/index.html"}),
            (r"/live", LiveHandler),
        ]
        settings = dict(
            debug=True,
            static_path="./static",
            template_path="./template",
        )
        cyclone.web.Application.__init__(self, handlers, **settings)


class StarWarsMixin(object):
    mbuffer = ""
    waiters = []

    def subscribe(self, client):
        StarWarsMixin.waiters.append(client)

    def unsubscribe(self, client):
        StarWarsMixin.waiters.remove(client)

    def broadcast(self, message):
        cls = StarWarsMixin

        chunks = (self.mbuffer + message.replace("\x1b[J", "")).split("\x1b[H")
        self.mbuffer = ""
        for chunk in chunks:
            chunksize = len(chunk)
            if len(chunk) == 985:
                log.msg("Sending new message to %r listeners" % len(cls.waiters))
                for client in cls.waiters:
                    try:
                        client.sendEvent(chunk.replace("\r\n", "<br>"))
                    except:
                        log.err()
            else:
                self.mbuffer = chunk


class LiveHandler(cyclone.sse.SSEHandler, StarWarsMixin):
    def bind(self):
        self.subscribe(self)

    def unbind(self):
        self.unsubscribe(self)


class BlinkenlightsProtocol(telnet.Telnet, StarWarsMixin):
    def dataReceived(self, data):
        self.broadcast(data)


def main():
    log.startLogging(sys.stdout)

    blinkenlights = protocol.ReconnectingClientFactory()
    blinkenlights.protocol = BlinkenlightsProtocol
    reactor.connectTCP("towel.blinkenlights.nl", 23, blinkenlights)

    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



# coding: utf-8
# Copyright 2009 Alexandre Fiori
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import types
from cyclone.redis import api
from cyclone.redis.protocol import RedisProtocol, SubscriberProtocol
from twisted.internet import defer, reactor, protocol


class RedisFactory(protocol.ReconnectingClientFactory):
    maxDelay = 10
    protocol = RedisProtocol

    def __init__(self, pool_size, db=0, isLazy=False):
        if not isinstance(db, types.IntType):
            raise ValueError("REDIS DB should be an integer: %s / %s" % (type(db), repr(db)))

        self.db = db
        self.idx = 0
        self.size = 0
        self.pool = []
        self.isLazy = isLazy
        self.pool_size = pool_size
        self.deferred = defer.Deferred()
        self.API = api.RedisAPI(self)

    def append(self, conn):
        self.pool.append(conn)
        self.size += 1
        if self.deferred and self.size == self.pool_size:
            self.deferred.callback(self.API)
            self.deferred = None

    def error(self, reason):
        if self.deferred:
            self.deferred.errback(ValueError(reason))
            self.deferred = None

    def remove(self, conn):
        try:
            self.pool.remove(conn)
        except:
            pass
        self.size = len(self.pool)

    @property
    def connection(self):
        assert self.size
        conn = self.pool[self.idx % self.size]
        self.idx += 1
        return conn


class SubscriberFactory(protocol.ReconnectingClientFactory):
    maxDelay = 120
    continueTrying = True
    protocol = SubscriberProtocol


def _Connection(host, port, reconnect, pool_size, db, lazy):
    factory = RedisFactory(pool_size, db, lazy)
    factory.continueTrying = reconnect
    for x in xrange(pool_size):
        reactor.connectTCP(host, port, factory)
    return (lazy is True) and factory.API or factory.deferred

def _ShardingConnection(hosts, reconnect, pool_size, db, lazy):
    err = "please use a list or tuple with host:port"
    if not isinstance(hosts, (types.ListType, types.TupleType)):
        raise ValueError(err)

    connections = []
    for item in hosts:
        try:
            host, port = item.split(":")
            port = int(port)
        except:
            raise ValueError(err)
        else:
            d = _Connection(host, port, reconnect, pool_size, db, lazy)
            connections.append(d)

    if lazy is True:
        return api.RedisShardingAPI(connections)
    else:
        deferred = defer.DeferredList(connections)
        api.RedisShardingAPI(deferred)
        return deferred


def RedisConnection(host="localhost", port=6379, reconnect=True, db=0):
    return _Connection(host, port, reconnect, pool_size=1, db=db, lazy=False)

def lazyRedisConnection(host="localhost", port=6379, reconnect=True, db=0):
    return _Connection(host, port, reconnect, pool_size=1, db=db, lazy=True)

def RedisConnectionPool(host="localhost", port=6379, reconnect=True, pool_size=5, db=0):
    return _Connection(host, port, reconnect, pool_size, db=db, lazy=False)

def lazyRedisConnectionPool(host="localhost", port=6379, reconnect=True, pool_size=5, db=0):
    return _Connection(host, port, reconnect, pool_size, db=db, lazy=True)

def RedisShardingConnection(hosts, reconnect=True, db=0):
    return _ShardingConnection(hosts, reconnect, pool_size=1, db=db, lazy=False)

def RedisShardingConnectionPool(hosts, reconnect=True, pool_size=5, db=0):
    return _ShardingConnection(hosts, reconnect, pool_size, db=db, lazy=False)

def lazyRedisShardingConnection(hosts, reconnect=True, db=0):
    return _ShardingConnection(hosts, reconnect, pool_size=1, db=db, lazy=True)

def lazyRedisShardingConnectionPool(hosts, reconnect=True, pool_size=5, db=0):
    return _ShardingConnection(hosts, reconnect, pool_size, db=db, lazy=True)



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
import functools
import sys

import cyclone.web

from twisted.python import log
from twisted.internet import reactor

class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", IndexHandler),
        ]
        cyclone.web.Application.__init__(self, handlers, debug=True)


def HTTPBasic(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        try:
            auth_type, auth_data = self.request.headers["Authorization"].split()
            assert auth_type == "Basic"
            usr, pwd = base64.b64decode(auth_data).split(":", 1)
            assert usr == "root@localhost"
            assert pwd == "123"
        except:
            # All arguments are optional. Defaults are:
            # log_message=None, auth_type="Basic", realm="Restricted Access"
            raise cyclone.web.HTTPAuthenticationRequired(
                                log_message="Authentication failed!",
                                auth_type="Basic", realm="DEMO")
        else:
            self._current_user = usr
            return method(self, *args, **kwargs)
    return wrapper


class IndexHandler(cyclone.web.RequestHandler):
    @HTTPBasic
    def get(self):
        self.write("Hi, %s." % self._current_user)


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

import cyclone.auth
import cyclone.escape
import cyclone.web

from twisted.python import log
from twisted.internet import reactor


class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", MainHandler),
            (r"/auth/login", AuthHandler),
            (r"/auth/logout", LogoutHandler),
        ]
        settings = dict(
            cookie_secret="32oETzKXQAGaYdkL5gEmGeJJFuYh7EQnp2XdTP1o/Vo=",
            debug=True,
            login_url="/auth/login",
        )
        cyclone.web.Application.__init__(self, handlers, **settings)


class BaseHandler(cyclone.web.RequestHandler):
    def get_current_user(self):
        user_json = self.get_secure_cookie("user")
        if not user_json: return None
        return cyclone.escape.json_decode(user_json)


class MainHandler(BaseHandler):
    @cyclone.web.authenticated
    def get(self):
        name = cyclone.escape.xhtml_escape(self.current_user["name"])
        self.write("Hello, " + name)
        self.write("<br><br><a href=\"/auth/logout\">Log out</a>")


class AuthHandler(BaseHandler, cyclone.auth.GoogleMixin):
    @cyclone.web.asynchronous
    def get(self):
        if self.get_argument("openid.mode", None):
            self.get_authenticated_user(self._on_auth)
            return
        self.authenticate_redirect()

    def _on_auth(self, user):
        if not user:
            raise cyclone.web.HTTPError(500, "Google auth failed")
        self.set_secure_cookie("user", cyclone.escape.json_encode(user))
        self.redirect("/")


class LogoutHandler(BaseHandler):
    def get(self):
        self.clear_cookie("user")
        self.redirect("/")


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os.path
import sys

import cyclone.mail
import cyclone.web

from twisted.python import log
from twisted.internet import defer, reactor

class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", cyclone.web.RedirectHandler, {"url": "/static/index.html"}),
            (r"/sendmail", SendmailHandler),
        ]
        settings = dict(
            debug=True,
            static_path="./static",
            template_path="./template",
            email_settings=dict(
                host="smtp.gmail.com",  # mandatory
                port=587,               # optional. default=25 or 587 for TLS
                tls=True,               # optional. default=False
                username="foo",         # optional. no default
                password="bar",         # optional. no default
            )
        )
        cyclone.web.Application.__init__(self, handlers, **settings)


class SendmailHandler(cyclone.web.RequestHandler):
    @defer.inlineCallbacks
    def post(self):
        to_addrs = self.get_argument("to_addrs").split(",")
        subject = self.get_argument("subject")
        message = self.get_argument("message")
        content_type = self.get_argument("content_type")

        # message may also be an html template:
        # message = self.render_string("email.html", name="foobar")

        msg = cyclone.mail.Message(
            from_addr="you@domain.com",
            to_addrs=to_addrs,
            subject=subject,
            message=message,
            mime=content_type,  # optional. default=text/plain
            charset="utf-8")    # optional. default=utf-8

        img_path = os.path.join(self.settings.static_path, "me.png")
        msg.attach(img_path, mime="image/png")

        txt_path = os.path.join(self.settings.static_path, "info.txt")
        msg.attach(txt_path, mime="text/plain", charset="utf-8")

        msg.attach("fake.txt", mime="text/plain", charset="utf-8",
                   content="this file is fake!")

        try:
            response = yield cyclone.mail.sendmail(
                                self.settings.email_settings, msg)
            self.render("response.html", title="Success", response=response)
        except Exception, e:
            self.render("response.html", title="Failure", response=str(e))


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()



# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import absolute_import, division, with_statement


class ObjectDict(dict):
    """Makes a dictionary behave like an object."""
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

    def __setattr__(self, name, value):
        self[name] = value


def import_object(name):
    """Imports an object by name.

    import_object('x.y.z') is equivalent to 'from x.y import z'.

    >>> import tornado.escape
    >>> import_object('tornado.escape') is tornado.escape
    True
    >>> import_object('tornado.escape.utf8') is tornado.escape.utf8
    True
    """
    parts = name.split('.')
    obj = __import__('.'.join(parts[:-1]), None, None, [parts[-1]], 0)
    return getattr(obj, parts[-1])


# Fake byte literal support:  In python 2.6+, you can say b"foo" to get
# a byte literal (str in 2.x, bytes in 3.x).  There's no way to do this
# in a way that supports 2.5, though, so we need a function wrapper
# to convert our string literals.  b() should only be applied to literal
# latin1 strings.  Once we drop support for 2.5, we can remove this function
# and just use byte literals.
if str is unicode:
    def b(s):
        return s.encode('latin1')
    bytes_type = bytes
else:
    def b(s):
        return s
    bytes_type = str


def doctests():
    import doctest
    return doctest.DocTestSuite()



# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
import hashlib
import struct
import cyclone
import cyclone.web
import cyclone.escape
from twisted.python import log


class WebSocketHandler(cyclone.web.RequestHandler):
    def __init__(self, application, request):
        cyclone.web.RequestHandler.__init__(self, application, request)
        self.application = application
        self.request = request
        self.transport = request.connection.transport
        self.ws_protocol = None;

    def headersReceived(self):
        pass

    def connectionMade(self, *args, **kwargs):
        pass

    def connectionLost(self, reason):
        pass

    def messageReceived(self, message):
        pass

    def sendMessage(self, message):
        if isinstance(message, dict):
            message = cyclone.escape.json_encode(message)
        if isinstance(message, unicode):
            message = message.encode("utf-8")
        assert isinstance(message, str)
        self.ws_protocol.sendMessage(message)

    def _rawDataReceived(self, data):
        self.ws_protocol.handleRawData(data)

    def _execute(self, transforms, *args, **kwargs):
        try:
            assert self.request.headers["Upgrade"].lower() == "websocket"
            assert self.request.headers["Connection"].lower() == "upgrade"
        except:
            return self.forbidConnection("Expected WebSocket Headers")

        if self.request.headers.has_key('Sec-Websocket-Version') and \
        self.request.headers['Sec-Websocket-Version'] in ('7', '8', '13'):
            self.ws_protocol  = WebSocketProtocol17(self)
        elif self.request.headers.has_key("Sec-WebSocket-Version"):
            self.transport.write(cyclone.escape.utf8(
                "HTTP/1.1 426 Upgrade Required\r\n"
                "Sec-WebSocket-Version: 8\r\n\r\n"))
            self.transport.loseConnection()
        else:
            self.ws_protocol = WebSocketProtocol76(self)

        self.request.connection.setRawMode()
        self.request.connection.rawDataReceived = self.ws_protocol.rawDataReceived
        self.ws_protocol.acceptConnection()

        self.connectionMade(*args, **kwargs)

    def forbidConnection(self, message):
        self.transport.write(
            "HTTP/1.1 403 Forbidden\r\nContent-Length: %s\r\n\r\n%s" % \
            (str(len(message)), message))
        return self.transport.loseConnection()


class WebSocketProtocol(object):
    def __init__(self, handler):
        self.handler = handler
        self.request = handler.request
        self.transport = handler.transport

    def acceptConnection(self):
        pass

    def rawDataReceived(self, data):
        pass

    def sendMessage(self, message):
        pass


class WebSocketProtocol17(WebSocketProtocol):
    def __init__(self, handler):
        WebSocketProtocol.__init__(self, handler)

        self._partial_data = None

        self._frame_fin = None
        self._frame_rsv = None
        self._frame_ops = None
        self._frame_mask = None
        self._frame_payload_length = None
        self._frame_header_length = None

        self._raw_data_length = None
        self._header_index = None

        self._message_buffer = ""


    def acceptConnection(self):
        log.msg('Using ws spec (draft 17)')

        # The difference between version 8 and 13 is that in 8 the
        # client sends a "Sec-Websocket-Origin" header and in 13 it's
        # simply "Origin".
        if 'Origin' in self.request.headers:
            origin = self.request.headers['Origin']
        else:
            origin = self.request.headers['Sec-Websocket-Origin']

        key = self.request.headers['Sec-Websocket-Key']
        accept = base64.b64encode(hashlib.sha1("%s%s" % \
            (key, '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')).digest())

        self.transport.write(
            "HTTP/1.1 101 Web Socket Protocol Handshake\r\n"
            "Upgrade: WebSocket\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Accept: %s\r\n"
            "Server: cyclone/%s\r\n"
            "WebSocket-Origin: %s\r\n"
            "WebSocket-Location: ws://%s%s\r\n\r\n" % \
            (accept, cyclone.version, origin,
             self.request.host, self.request.path))

    def rawDataReceived(self, data):
        self._raw_data_len = len(data)

        if self._partial_data:
            data = self._partial_data + data
            self._partial_data = None

        self._processFrameHeader(data)

        if (self._raw_data_len - self._header_index) < self._frame_payload_len:
            self._partial_data = data
            return

        self._message_buffer += self._extractMessageFromFrame(data)
        if self._frame_fin:
            self.handler.messageReceived(self._message_buffer)
            self._message_buffer = ""

        # if there is still data after this frame, process again
        current_len = self._frame_header_len + self._frame_payload_len
        if current_len < self._raw_data_len:
            self.rawDataReceived(data[current_len:])

    def _processFrameHeader(self, data):
        # first byte contains fin, rsv and ops
        b = ord(data[0])
        self._frame_fin = (b & 0x80) != 0
        self._frame_rsv = (b & 0x70) >> 4
        self._frame_ops = b & 0x0f

        # second byte contains mask and payload length
        b = ord(data[1])
        self._frame_mask = (b & 0x80) != 0
        frame_payload_len1 = b & 0x7f

        if (self._frame_mask):
            mask_len = 4
        else:
            mask_len = 0

        # i is frame index. It's at 2 here because we've already processed the
        # first 2 bytes of the frame.
        i = 2

        if frame_payload_len1 <  126:
            self._frame_header_len = i + mask_len
            self._frame_payload_len = frame_payload_len1
        elif frame_payload_len1 == 126:
            self._frame_header_len = i + 2 + mask_len
            self._frame_payload_len = struct.unpack("!H", data[i:i+2])[0]
            i += 2
        elif frame_payload_len1 == 127:
            self._frame_header_len = i + 8 + mask_len
            self._frame_payload_len = struct.unpack("!Q", data[i:i+8])[0]
            i += 8

        self._header_index = i

    def _extractMessageFromFrame(self, data):
        i = self._header_index

        # when payload is masked, extract frame mask
        frame_mask = None
        frame_mask_array = []
        if self._frame_mask:
            frame_mask = data[i:i+4]
            for j in range(0, 4):
                frame_mask_array.append(ord(frame_mask[j]))
            i += 4
            payload = bytearray(data[i:i+self._frame_payload_len])
            for k in xrange(0, self._frame_payload_len):
                payload[k] ^= frame_mask_array[k % 4]

            return str(payload)

    def sendMessage(self, message):
        message = unicode(message, "utf-8")
        length = len(message)
        newFrame = []
        newFrame.append(0x81)
        newFrame = bytearray(newFrame)
        if length <= 125:
            newFrame.append(length)
        elif length > 125 and length < 65536:
            newFrame.append(126)
            newFrame += struct.pack('!H', length)
        elif length >= 65536:
            newFrame.append(127)
            newFrame += struct.pack('!Q', length)

        newFrame += message.encode('utf-8')
        self.transport.write(str(newFrame))


class WebSocketProtocol76(WebSocketProtocol):
    def __init__(self, handler):
        WebSocketProtocol.__init__(self, handler)

        self._k1 = None
        self._k2 = None
        self._nonce = None

        self._postheader = False
        self._protocol = None

    def acceptConnection(self):
        if self.request.headers.has_key('Sec-Websocket-Key1') == False or \
            self.request.headers.has_key('Sec-Websocket-Key2') == False:
            log.msg('Using old ws spec (draft 75)')
            self.transport.write(
                "HTTP/1.1 101 Web Socket Protocol Handshake\r\n"
                "Upgrade: WebSocket\r\n"
                "Connection: Upgrade\r\n"
                "Server: cyclone/%s\r\n"
                "WebSocket-Origin: %s\r\n"
                "WebSocket-Location: ws://%s%s\r\n\r\n" % \
                (cyclone.version, self.request.headers["Origin"],
                 self.request.host, self.request.path))
            self._protocol = 75
        else:
            log.msg('Using ws draft 76 header exchange')
            self._k1 = self.request.headers["Sec-WebSocket-Key1"]
            self._k2 = self.request.headers["Sec-WebSocket-Key2"]
            self._protocol = 76
        self._postheader = True

    def rawDataReceived(self, data):
        if self._postheader == True and self._protocol >= 76 and len(data) == 8:
            self._nonce = data.strip()
            token = self._calculate_token(self._k1, self._k2, self._nonce)
            self.transport.write(
                "HTTP/1.1 101 Web Socket Protocol Handshake\r\n"
                "Upgrade: WebSocket\r\n"
                "Connection: Upgrade\r\n"
                "Server: cyclone/%s\r\n"
                "Sec-WebSocket-Origin: %s\r\n"
                "Sec-WebSocket-Location: ws://%s%s\r\n\r\n%s\r\n\r\n" % \
                (cyclone.version, self.request.headers["Origin"],
                 self.request.host, self.request.path, token))
            self._postheader = False
            self.handler.flush()
            return

        try:
            messages = data.split('\xff')
            for message in messages[:-1]:
                self.handler.messageReceived(message[1:])
        except Exception, e:
            log.msg("Invalid WebSocket Message: %s" % repr(data))
            self._handle_request_exception(e)

    def sendMessage(self, message):
        self.transport.write("\x00" + message + "\xff")

    def _calculate_token(self, k1, k2, k3):
        token = struct.pack('>II8s', self._filterella(k1), self._filterella(k2), k3)
        return hashlib.md5(token).digest()

    def _filterella(self, w):
        nums = []
        spaces = 0
        for l in w:
            if l.isdigit(): nums.append(l)
            if l.isspace(): spaces = spaces + 1
        x = int(''.join(nums))/spaces
        return x



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2010 Alexandre Fiori
# based on the original Tornado by Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os.path
import sys

import cyclone.locale
import cyclone.web

from twisted.python import log
from twisted.internet import reactor

class Application(cyclone.web.Application):
    def __init__(self):
        handlers = [
            (r"/", IndexHandler),
        ]
        settings = dict(
            debug=True,
            template_path="./frontend/template",
        )
        cyclone.locale.load_gettext_translations("./frontend/locale", "mytest")
        cyclone.web.Application.__init__(self, handlers, **settings)


class BaseHandler(cyclone.web.RequestHandler):
    def get_user_locale(self):
        lang = self.get_cookie("lang", default="en_US")
        return cyclone.locale.get(lang)


class IndexHandler(BaseHandler):
    def _apples(self):
        try:
            return int(self.get_argument("apples", 1))
        except:
            return 1

    def get(self):
        self.render("index.html",
                    apples=self._apples(),
                    locale=self.locale.code,
                    languages=cyclone.locale.get_supported_locales())

    def post(self):
        lang = self.get_argument("lang")
        # assert lang in cyclone.locale.get_supported_locales()

        # Either set self._locale or override get_user_locale()
        #   self._locale = cyclone.locale.get(lang)
        #   self.render(...)

        self.set_cookie("lang", lang)
        self.redirect("/?apples=%d" % self._apples())


def main():
    log.startLogging(sys.stdout)
    reactor.listenTCP(8888, Application(), interface="127.0.0.1")
    reactor.run()


if __name__ == "__main__":
    main()
